@@ -31,6 +31,7 @@
 #include <js/TypeDecls.h>
 #include <js/Warnings.h>
 
+#include "gi/arg-inl.h"
 #include "gi/function.h"
 #include "gi/repo.h"
 #include "gi/union.h"
@@ -52,7 +53,7 @@ UnionInstance::UnionInstance(JSContext* cx, JS::HandleObject obj)
 
 UnionInstance::~UnionInstance(void) {
     if (m_ptr) {
-        g_boxed_free(g_registered_type_info_get_g_type(info()), m_ptr);
+        g_boxed_free(gtype(), m_ptr);
         m_ptr = nullptr;
     }
     GJS_DEC_COUNTER(union_instance);
@@ -76,9 +77,7 @@ bool UnionPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
                       method_info.name(), ns(), name());
 
             /* obj is union proto */
-            if (!gjs_define_function(context, obj,
-                                     g_registered_type_info_get_g_type(info()),
-                                     method_info))
+            if (!gjs_define_function(context, obj, gtype(), method_info))
                 return false;
 
             *resolved = true; /* we defined the prop in object_proto */
@@ -115,7 +114,7 @@ static void* union_new(JSContext* context, JS::HandleObject this_obj,
                                                args, &rval))
                 return nullptr;
 
-            if (!rval.v_pointer) {
+            if (!gjs_arg_get<void*>(&rval)) {
                 gjs_throw(context,
                           "Unable to construct union type %s as its"
                           "constructor function returned null",
@@ -123,7 +122,7 @@ static void* union_new(JSContext* context, JS::HandleObject this_obj,
                 return nullptr;
             }
 
-            return rval.v_pointer;
+            return gjs_arg_get<void*>(&rval);
         }
     }
 
