@@ -37,7 +37,7 @@ function _init() {
     let {GtkWidgetClass} = Legacy.defineGtkLegacyObjects(GObject, Gtk);
     Gtk.Widget.prototype.__metaclass__ = GtkWidgetClass;
 
-    if (Gtk.Container.prototype.child_set_property) {
+    if (Gtk.Container && Gtk.Container.prototype.child_set_property) {
         Gtk.Container.prototype.child_set_property = function (child, property, value) {
             GjsPrivate.gtk_container_child_set_property(this, child, property, value);
         };
@@ -125,6 +125,13 @@ function _init() {
         return klass;
     };
 
+    if (Gtk.Widget.prototype.get_first_child) {
+        Gtk.Widget.prototype[Symbol.iterator] = function* () {
+            for (let c = this.get_first_child(); c; c = c.get_next_sibling())
+                yield c;
+        };
+    }
+
     if (Gtk.BuilderScope) {
         BuilderScope = GObject.registerClass({
             Implements: [Gtk.BuilderScope],
