@@ -36,17 +36,18 @@ FWD_HEADER = '#include <js/TypeDecls.h>'
 FWD_DECLS_IN_HEADER = (
     'class JSAtom;',
     'struct JSContext;',
+    'struct JSClass;',
     'class JSFunction;',
+    'class JSFreeOp;',
     'class JSObject;',
     'struct JSRuntime;',
     'class JSScript;',
     'class JSString;',
-    'struct JSFreeOp;',
     'namespace js { class TempAllocPolicy; }'
     'namespace JS { struct PropertyKey; }',
     'namespace JS { class Symbol; }',
     'namespace JS { class BigInt; }',
-    'namespace JS { union Value; }',
+    'namespace JS { class Value; }',
     'namespace JS { class Compartment; }',
     'namespace JS { class Realm; }',
     'namespace JS { struct Runtime; }',
@@ -65,6 +66,10 @@ FALSE_POSITIVES = (
     ('gi/private.cpp', '#include <algorithm>', 'for max'),
     ('gjs/importer.cpp', '#include <algorithm>', 'for max'),
     ('modules/cairo-context.cpp', '#include <algorithm>', 'for max'),
+
+    # Weird false positive on some versions of IWYU
+    ('gi/arg.cpp', 'struct _GHashTable;', ''),
+    ('gi/arg.cpp', 'struct _GVariant;', ''),
 )
 
 
